@@ -8,6 +8,7 @@ import sys
 import traceback
 import yaml
 import fasttext
+from fastspell import FastSpell
 
 from heapq import heappush, heappop
 from multiprocessing import Queue, Process, Value, cpu_count
@@ -29,6 +30,8 @@ except (SystemError, ImportError):
 __author__ = "Sergio Ortiz Rojas"
 __version__ = "Version 1.0 # 24/05/2021 # Separate hardrules package from Bicleaner # Jaume Zaragoza"
 __version__ = "Version 1.1 # 26/05/2021 # Load lm only when necessary # Jaume Zaragoza"
+__version__ = "Version 1.3 # 05/07/2021 # FastSpell, bad encoding Ä, check identical with alphabetic and discard empty sides # Jaume Zaragoza"
+__version__ = "Version 1.3.1 # 12/07/2021 # Fix PyPi release # Jaume Zaragoza"
 
 logging_level = 0
 
@@ -234,6 +237,13 @@ def worker_process(i, jobs_queue, output_queue, args):
     else:
         porn_removal = None
         porn_tokenizer = None
+        
+    if not args.disable_lang_ident:
+        fastspell_src = FastSpell.FastSpell(args.source_lang, mode="cons")
+        fastspell_trg = FastSpell.FastSpell(args.target_lang, mode="cons")
+    else:
+        fastspell_src = None
+        fastspell_trg = None    
 
     while True:
         job = jobs_queue.get()
@@ -252,10 +262,10 @@ def worker_process(i, jobs_queue, output_queue, args):
                     if len(parts) >=  args.scol and len(parts) >= args.tcol:
                         left = parts[args.scol-1]
                         right = parts[args.tcol-1]
+                        wrong_tu_results = wrong_tu(left,right, args, lm_filter, porn_removal, porn_tokenizer, fastspell_src, fastspell_trg)
                     else:
-                        logging.error("WARNING: scol ({}) or tcol ({}) indexes above column number ({})".format(args.scol, args.tcol, len(parts)))        
-                        continue
-                    wrong_tu_results = wrong_tu(left,right, args, lm_filter, porn_removal, porn_tokenizer)
+                        logging.error("scol ({}) or tcol ({}) indexes above column number ({})".format(args.scol, args.tcol, len(parts)))
+                        wrong_tu_results = "c_wrong_cols"
 
                     # Print input sentences when scoring_only is disabled
                     if not args.score_only:
@@ -352,8 +362,11 @@ def perform_hardrules_filtering(args):
 
     logging.info("End mapping")
 
+    errors = False
     for w in workers:
         w.join()
+        if w.exitcode != 0:
+            errors = True
 
     # Reducer termination
     output_queue.put(None)
@@ -367,10 +380,16 @@ def perform_hardrules_filtering(args):
     logging.info("Elapsed time {0:.2f} s".format(elapsed_time))
     logging.info("Troughput: {0} rows/s".format(int((nline*1.0)/elapsed_time)))
 
+    return errors
+
 def main(args):
     logging.info("Executing main program...")
-    perform_hardrules_filtering(args)
-    logging.info("Program finished")
+    errors = perform_hardrules_filtering(args)
+    if errors:
+        logging.error("Program finished with errors")
+        sys.exit(1)
+    else:
+        logging.info("Program finished")
 
 if __name__ == '__main__':
     try: 
