@@ -1,9 +1,12 @@
-import pycld2
+import unicodedata
+import logging
 import regex
+import sys
+from fastspell import FastSpell
 
+tbl_non_alpha = [chr(i) for i in range(sys.maxunicode) if not unicodedata.category(chr(i)).startswith('L')]
+tbl_non_alpha = str.maketrans('', '', ''.join(tbl_non_alpha))
 regex_blank = regex.compile("[ \u00A0]")
-regex_digit = regex.compile("[[:digit:]]")
-regex_punct = regex.compile("[[:punct:]]")
 regex_alpha = regex.compile("[[:alpha:]]")
 regex_url = regex.compile('((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]|\((:?[^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?\xab\xbb\u201c\u201d\u2018\u2019]))')
 #regex_breadcrumbs = regex.compile("([ ][-/»][ ]|[|<>→←]|[ ][:][:][ ])")
@@ -20,37 +23,25 @@ regex_glued_words = regex.compile("([[:alpha:]]*[[:upper:]]{1}[[:lower:]]+){3}")
 safe_noise_detection_langs = {"en", "es", "fr", "pl", "de", "it", "pt", "nl", "cs", "ro", "fi", "lv", "et", "bg", "hr", "da", "hu", "ga", "eu", "gl", "sl", "sv", "mt", "sk"}
 
 safe_noise_detection_langs = {"en", "es", "fr", "pl", "de", "it", "pt", "nl", "cs", "ro", "fi", "lv", "et", "bg", "hr", "da", "hu", "ga", "eu", "gl", "sl", "sv", "mt", "sk", "is", "lt", "nb", "nn", "no"}
-similar_pairs = [{"es","ca"}, {"es","gl"}, {"pt","gl"}, {"no","nn"}, {"no", "da"}]
+#similar_pairs = [{"es","ca"}, {"es","gl"}, {"pt","gl"}, {"no","nn"}, {"no", "da"}]
+atilde_langs = {"pt"}
+acumflex_langs = {"cy", "fr", "fa", "it", "pt", "tr", "vi",}
 
-def c_identical(left, right, left_lang, right_lang):
-    if left_lang =="nb":
-        left_lang="no"
-    if right_lang=="nb":
-        right_lang="no"
-#    if ({left_lang, right_lang} in similar_pairs):        
-#        return True
+def c_identical_alpha(left, right):
+    left = left.translate(tbl_non_alpha)
+    right = right.translate(tbl_non_alpha)
     return left.casefold() != right.casefold()
-    
-def c_identical_wo_digits(left, right, left_lang, right_lang):
-    left = regex_digit.sub("", left)
-    right = regex_digit.sub("", right)
-    return c_identical(left, right, left_lang, right_lang)
 
-def c_identical_wo_punct(left, right, left_lang, right_lang):
-    left = regex_punct.sub("", left)
-    right = regex_punct.sub("", right)
-    return c_identical(left, right, left_lang, right_lang)
-        
 def c_minimal_length(sentence):
     """ Counts number of whitespace, requires >= 2 (3 words) """
     return len(regex_blank.findall(sentence)) >= 2
-        
+
 def c_length(left, right):
     return 0.5 <= float(len(left))/float(len(right)) <= 2.0
 
 def c_length_bytes(left, right):
     return 0.5 <= float(len(left.encode("utf8")))/float(len(right.encode("utf8"))) <= 2.0
-
+'''
 def c_different_language(left, right, left_lang, right_lang):
     if left_lang =="nb":
         left_lang="no"
@@ -88,7 +79,8 @@ def c_different_language(left, right, left_lang, right_lang):
             return True
         else:    
             return False
-        
+'''
+'''        
 def c_reliable_long_language(sentence, language):
     if language=="nb":
         language = "no"
@@ -109,7 +101,14 @@ def c_reliable_long_language(sentence, language):
             return False
     else:
         return True
-        
+'''
+def c_no_bad_encoding(sentence, lang):
+    if lang not in atilde_langs and 'Ã' in sentence:
+        return False
+    if lang not in acumflex_langs and 'Â' in sentence:
+        return False
+    return True
+
 def c_alpha(sentence):
     return len(regex_alpha.findall(sentence)) > 0
     
@@ -160,7 +159,11 @@ def c_no_porn(left, right, model, side, porn_tokenizer):
         tok = porn_tokenizer.tokenize(right.lower())
     return model.predict(porn_tokenizer.detokenize(tok))[0][0] == '__label__negative'
 
-def wrong_tu(left, right, args, lm_filter = None, porn_removal = None, porn_tokenizer = None):
+def wrong_tu(left, right, args, lm_filter = None, porn_removal = None, porn_tokenizer = None, fastspell_src = None, fastspell_trg = None):
+    if not left:
+        return "c_no_empty(left)"
+    if not right:
+        return "c_no_empty(right)"
     if len(left) >= 1024:
         return "len(left) >= 1024"
     if len(right) >= 1024:
@@ -173,14 +176,10 @@ def wrong_tu(left, right, args, lm_filter = None, porn_removal = None, porn_toke
         return "c_minimal_length(left) and c_minimal_length(right)"
     elif not (c_length(left, right) or c_length_bytes(left, right)): 
         return "c_length or c_length_bytes"
-    elif not c_identical(left, right, args.source_lang, args.target_lang):
-        return "c_identical"
-    elif not c_identical_wo_digits(left, right, args.source_lang, args.target_lang):
-        return "c_identical_wo_digits"    
-    elif not c_identical_wo_punct(left, right, args.source_lang, args.target_lang):
-        return "c_identical_wo_punct"    
-    elif (not args.disable_lang_ident and not  c_different_language(left, right, args.source_lang, args.target_lang)):
-        return "c_different_language"
+    elif not c_identical_alpha(left, right):
+        return "c_identical_alpha"
+#    elif (not args.disable_lang_ident and not  c_different_language(left, right, args.source_lang, args.target_lang)):
+#        return "c_different_language"
     elif not c_majority_alpha(left):
         return "c_majority_alpha(left)"
     elif not c_majority_alpha(right):
@@ -233,12 +232,18 @@ def wrong_tu(left, right, args, lm_filter = None, porn_removal = None, porn_toke
         return 'c_no_literals(["{{", "%s", "}}"], left)'
     elif not c_no_literals(["{{", "%s", "}}"], right):
         return 'c_no_literals(["{{", "%s", "}}"], right)'
+    elif not c_no_bad_encoding(left, args.source_lang) and not c_no_bad_encoding(right, args.target_lang):
+        return 'c_no_bad_encoding(["Â","Ã"])'
     elif left.istitle() and right.istitle():
         return 'left.istitle() and right.istitle()'
-    elif (not args.disable_lang_ident and not  c_reliable_long_language(left, args.source_lang)):
-        return "c_reliable_long_language(left, sourcelang)"
-    elif (not args.disable_lang_ident and  not c_reliable_long_language(right, args.target_lang)):
-        return "c_reliable_long_language(right, targetlang)"
+#    elif (not args.disable_lang_ident and not  c_reliable_long_language(left, args.source_lang)):
+#        return "c_reliable_long_language(left, sourcelang)"
+#    elif (not args.disable_lang_ident and  not c_reliable_long_language(right, args.target_lang)):
+#        return "c_reliable_long_language(right, targetlang)"
+    elif (not args.disable_lang_ident and not  fastspell_src.getlang(left)==args.source_lang):
+        return "c_wrong_language(left, sourcelang)"
+    elif (not args.disable_lang_ident and  not fastspell_trg.getlang(right)==args.target_lang):
+        return "c_wrong_language(right, targetlang)"
     elif not args.disable_porn_removal and porn_removal != None and not c_no_porn(left, right, porn_removal, args.metadata_yaml['porn_removal_side'], porn_tokenizer):
         return "c_no_porn"
     elif  args.disable_lm_filter == False and lm_filter != None and lm_filter.score(left, right) < args.lm_threshold:
